import * as vscode from 'vscode-languageserver';
import * as fs from './fs';
export declare namespace GlobalRefsRequest {
    const type: vscode.RequestType<vscode.WorkspaceSymbolParams, vscode.SymbolInformation[], any>;
}
export declare namespace InitializeRequest {
    const type: vscode.RequestType<vscode.InitializeParams, InitializeResult, any>;
}
export interface InitializeResult extends vscode.InitializeResult {
    /**
     * The capabilities the language server provides.
     */
    capabilities: ServerCapabilities;
}
export interface ServerCapabilities extends vscode.ServerCapabilities {
    xworkspaceReferencesProvider?: boolean;
    xdefinitionProvider?: boolean;
    xdependenciesProvider?: boolean;
    xpackagesProvider?: boolean;
}
export declare namespace ShutdownRequest {
    const type: {
        readonly method: string;
    };
}
export declare namespace ExitRequest {
    const type: {
        readonly method: string;
    };
}
export declare namespace ReadDirRequest {
    const type: vscode.RequestType<string, fs.FileInfo[], any>;
}
export declare namespace ReadFileRequest {
    const type: vscode.RequestType<string, string, any>;
}
export declare namespace DefinitionRequest {
    const type: vscode.RequestType<vscode.TextDocumentPositionParams, vscode.Location[], any>;
}
export declare namespace XdefinitionRequest {
    const type: vscode.RequestType<vscode.TextDocumentPositionParams, SymbolLocationInformation[], any>;
}
export declare namespace HoverRequest {
    const type: vscode.RequestType<vscode.TextDocumentPositionParams, vscode.Hover, any>;
}
export declare namespace ReferencesRequest {
    const type: vscode.RequestType<vscode.ReferenceParams, vscode.Location[], any>;
}
export declare namespace DependenciesRequest {
    const type: vscode.RequestType<void, DependencyReference[], any>;
}
export declare namespace PackagesRequest {
    const type: vscode.RequestType<void, PackageInformation[], any>;
}
export declare namespace WorkspaceSymbolsRequest {
    const type: vscode.RequestType<WorkspaceSymbolParams, vscode.SymbolInformation[], any>;
}
export declare namespace WorkspaceReferenceRequest {
    const type: vscode.RequestType<WorkspaceReferenceParams, ReferenceInformation[], any>;
}
export interface SymbolDescriptor {
    kind: string;
    name: string;
    containerKind: string;
    containerName: string;
    package?: PackageDescriptor;
}
export interface PartialSymbolDescriptor {
    kind?: string;
    name?: string;
    containerKind?: string;
    containerName?: string;
    package?: PackageDescriptor;
}
export declare namespace SymbolDescriptor {
    function create(kind: string, name: string, containerKind: string, containerName: string, pkg?: PackageDescriptor): SymbolDescriptor;
}
export interface WorkspaceSymbolParams {
    /**
     * A non-empty query string.
     */
    query?: string;
    /**
     * A set of properties that describe the symbol to look up.
     */
    symbol?: PartialSymbolDescriptor;
    /**
     * The number of items to which to restrict the results set size.
     */
    limit: number;
}
export interface WorkspaceReferenceParams {
    query: PartialSymbolDescriptor;
    hints?: DependencyHints;
}
export interface SymbolLocationInformation {
    location?: vscode.Location;
    symbol: SymbolDescriptor;
}
export interface ReferenceInformation {
    reference: vscode.Location;
    symbol: SymbolDescriptor;
}
export interface PackageInformation {
    package: PackageDescriptor;
    dependencies: DependencyReference[];
}
export interface PackageDescriptor {
    name: string;
    version?: string;
    repoURL?: string;
}
export interface DependencyHints {
    dependeePackageName?: string;
}
export interface DependencyReference {
    attributes: PackageDescriptor;
    hints: DependencyHints;
}
export declare namespace DocumentSymbolRequest {
    const type: vscode.RequestType<vscode.DocumentSymbolParams, vscode.SymbolInformation[], any>;
}
export declare namespace TextDocumentDidOpenNotification {
    const type: vscode.NotificationType<vscode.DidOpenTextDocumentParams>;
}
export declare namespace TextDocumentDidCloseNotification {
    const type: vscode.NotificationType<vscode.DidCloseTextDocumentParams>;
}
export declare namespace TextDocumentDidSaveNotification {
    const type: vscode.NotificationType<vscode.DidSaveTextDocumentParams>;
}
export declare namespace TextDocumentDidChangeNotification {
    const type: vscode.NotificationType<vscode.DidChangeTextDocumentParams>;
}
export declare namespace TextDocumentCompletionRequest {
    const type: vscode.RequestType<vscode.TextDocumentPositionParams, vscode.CompletionList, void>;
}
