"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const net = require("net");
const cluster = require("cluster");
const connection_1 = require("./connection");
const master_connection_1 = require("./master-connection");
const request_type_1 = require("./request-type");
function rewriteConsole() {
    return __awaiter(this, void 0, void 0, function* () {
        const consoleErr = console.error;
        console.error = function () {
            if (cluster.isMaster) {
                consoleErr(`[mstr]`, ...arguments);
            }
            else {
                consoleErr(`[wkr${cluster.worker.id}]`, ...arguments);
            }
        };
    });
}
/**
 * serve starts a singleton language server instance that uses a
 * cluster of worker processes to achieve some semblance of
 * parallelism.
 *
 * @param createWorkerConnection Should return a connection to a subworker (for example by spawning a child process)
 */
function serve(options, createLangHandler, createWorkerConnection) {
    return __awaiter(this, void 0, void 0, function* () {
        rewriteConsole();
        if (cluster.isMaster) {
            console.error(`Master node process spawning ${options.clusterSize} workers`);
            for (let i = 0; i < options.clusterSize; ++i) {
                const worker = cluster.fork().on('disconnect', () => {
                    console.error(`worker ${worker.process.pid} disconnect`);
                });
            }
            cluster.on('exit', (worker, code, signal) => {
                const reason = code === null ? signal : code;
                console.error(`worker ${worker.process.pid} exit (${reason})`);
            });
        }
        else {
            console.error('Listening for incoming LSP connections on', options.lspPort);
            var server = net.createServer(socket => {
                // This connection listens on the socket
                const master = connection_1.newConnection(socket, socket, options);
                // Override the default exit notification handler so the process is not killed
                master.onNotification(request_type_1.ExitRequest.type, () => {
                    console.error('Closing socket');
                    socket.end();
                });
                if (createWorkerConnection) {
                    // Spawn two child processes that communicate through STDIN/STDOUT
                    // One gets short-running requests like textDocument/definition,
                    // the other long-running requests like textDocument/references
                    // TODO: Don't spawn new processes on every connection, keep them warm
                    //       Need to make sure exit notifications don't come through and LS supports re-initialization
                    const leightWeightWorker = createWorkerConnection();
                    const heavyDutyWorker = createWorkerConnection();
                    master_connection_1.registerMasterHandler(master, leightWeightWorker, heavyDutyWorker);
                    leightWeightWorker.listen();
                    heavyDutyWorker.listen();
                }
                else {
                    connection_1.registerLanguageHandler(master, options.strict, createLangHandler());
                }
                master.listen();
            });
            server.listen(options.lspPort);
        }
    });
}
exports.serve = serve;
//# sourceMappingURL=server.js.map